//
// dengen control
// 2023.3.23
// 2023.3.17
// 2023.3.12
// 2023.3.10
// 2023.2.28
//
//                        ---------
//(thermistor) --> RA2/AN2|1    18| RA1/AN1 <-- analog 1/3 of output
//  relay ctrl --> RA3/AN3|2    17| RA0/AN0 <-- analog potentional meter
//             /-- RA4/AN4|3    16| RA7     --/ 
//             /-- RA5    |4    15| RA6     --/
//                     GND|5    14| +5V
//      com1 <--   RB0    |6    13| RB7/AN6 --> (fan PWM)
//      com2 <--   RB1    |7    12| RB6/AN5 --> SRCLK
//      com3 <--   RB2    |8    11| RB5     --> RCLK
//      com4 <--   RB3    |9    10| RB4     --> SER
//                        ---------

#include <xc.h>
#include <stdbool.h>
#include <string.h>

//#include "16F88.h
//#include P16F88.INC


//#define debug	1


// #pragma config  = 0x3F50
// CONFIG1
#pragma config FOSC = INTOSCIO  // Oscillator Selection bits (INTRC oscillator; port I/O function on both RA6/OSC2/CLKO pin and RA7/OSC1/CLKI pin)
#pragma config WDTE = OFF       // Watchdog Timer Enable bit (WDT disabled)
#pragma config PWRTE = ON       // Power-up Timer Enable bit (PWRT enabled)
#pragma config MCLRE = OFF      // RA5/MCLR/VPP Pin Function Select bit (RA5/MCLR/VPP pin function is digital I/O, MCLR internally tied to VDD)
#pragma config BOREN = ON       // Brown-out Reset Enable bit (BOR enabled)
#pragma config LVP = OFF        // Low-Voltage Programming Enable bit (RB3 is digital I/O, HV on MCLR must be used for programming)
#pragma config CPD = OFF        // Data EE Memory Code Protection bit (Code protection off)
#pragma config WRT = OFF        // Flash Program Memory Write Enable bits (Write protection off)
#pragma config CCPMX = RB0      // CCP1 Pin Selection bit (CCP1 function on RB0)
#pragma config CP = OFF         // Flash Program Memory Code Protection bit (Code protection off)

// CONFIG2
#pragma config FCMEN = ON       // Fail-Safe Clock Monitor Enable bit (Fail-Safe Clock Monitor enabled)
#pragma config IESO = ON        // Internal External Switchover bit (Internal External Switchover mode enabled)



#define Amode 0x07		//port A initial mode  7,6,5,4,3=output  2,1,0=input
#define Bmode 0x00		//port B initial mode  7,6,5,4,3,2,1,0=output

//-----------------------------------
#define ERROR			0
#define TRUE			1
#define FALSE			0
#define	HIGH			1
#define LOW				0
//-----------------------------------
#define RELAY	PORTAbits.RA3		// relay control
#define NOTUSE6	PORTAbits.RA6		// 
#define NOTUSE5	PORTAbits.RA5		// Not Use. RA5=input only
#define NOTUSE4	PORTAbits.RA4		// 
#define NOTUSE3	PORTAbits.RA3		// 
#define THERM	PORTAbits.RA2		// thermistor
#define VOL		PORTAbits.RA1		// analog 1/3 output voltage
#define POT		PORTAbits.RA0		// analog potentional meter

//-----------------------------------
#define FAN		PORTBbits.RB7		//
#define SRCLK	PORTBbits.RB6		//
#define RCLK	PORTBbits.RB5		//
#define SER		PORTBbits.RB4		//
#define COM4	PORTBbits.RB3		//
#define COM3	PORTBbits.RB2		//
#define COM2	PORTBbits.RB1		//
#define COM1	PORTBbits.RB0		//

//-----------------------------------
#define TIMER_INTERVAL (0xFFFF - 8000) // TMRW[̃JEgl
                                        // 4MHz/4=1MHz, 1/1vXP[ 1us x 8000 = 8msec

//-----------------------------------

void ad( unsigned int );
void convert2volt(void);
void get_val(void);

void delay_ms( unsigned long );

//-----------------------------------------------global
unsigned long	ch_v;
bool complete_status;
unsigned int	val4,val3,val2,val1;

unsigned int	leddata[]= {0xC0,0xF9,0xA4,0xB0,0x99,0x92,0x82,0xD8,0x80,0x90};

//----------------------------
void get_val(void){
unsigned int tmp;

	if( ch_v > 1000 ){      // bigger than 10V 1234
		val4 = ch_v/1000;		// 1
		tmp = val4 * 1000;		// 1000
		val3 = ch_v - tmp;		// 1234-1000=234
		val3 = val3 / 100;	// 2
		ch_v = ch_v - tmp;		// 1234-1000=234
	}
	else{
		val4 = 0;
		val3 = ch_v / 100;		// 2
	}

	tmp     = val3 * 100;		// 200
	val2  = ch_v - tmp;		// 234-200=34
	val2  = val2 / 10;		// 3
	ch_v = ch_v - tmp;			// 234-200=34

	tmp     = val2 * 10;		// 30
	val1   = ch_v - tmp;		// 34-30=4

}

//-----------------convert A/D data to numeral 500-0 ( 5.00V - 0.00V )
void convert2volt(void){
unsigned long	ad_ch;

	ad_ch = (ADRESH<<8)+ADRESL;
	
	ch_v = ad_ch * 500;
	ch_v = ch_v / 1024;			// 500 - 0

}

//-----------------LED data serial output
void serialout( unsigned int data ){
unsigned int i;

//	SRCLK = LOW;
//	RCLK = LOW;

	for(i=0; i<=8; i++){
		SER = (data >> 8) & 1;		// set MSB of data to SER
		SRCLK = HIGH;
		data = data << 1;
		SRCLK = LOW;
	}
//	RCLK = HIGH;
//	RCLK = LOW;

}

//-----------------LED data latch
void seriallatch( void ){

	RCLK = HIGH;
	RCLK = LOW;
}

//-----------------A/D convert ch
void ad( unsigned int ch ){

	switch( ch ){
		case 2:
			CHS2 = 0;
			CHS1 = 1;
			CHS0 = 0;
			break;
		case 1:
			CHS2 = 0;
			CHS1 = 0;
			CHS0 = 1;
			break;
		case 0:
			CHS2 = 0;
			CHS1 = 0;
			CHS0 = 0;
			break;
	}	// end of switch
	delay_ms(1);		//must, acquisition time
	GO = 1;
#ifndef debug
	while(GO);	//ADϊ
#endif

}

//-----------------------------------------------
void delay_ms( unsigned long millisec )
// Delays a multiple of 1 milliseconds at 4 MHz
// using the TMR0 timer
{

    char next = 0;

#ifdef debug
	return;
#endif


//    OPTION = 2; // prescaler divide TMR0 rate by 8
	PS2=0;
	PS1=1;
	PS0=0;
	TMR0 = 2;  // deduct 2*8 fixed instruction cycles delay
    do  {
        next += 125;
//        clrwdt();  // needed only if watchdog is enabled
        while (TMR0 != next)   // 125 * 8 = 1000 (= 1 ms)
            ;
    } while ( -- millisec != 0);
}

//----------------------------------------------------
void main(void)
{
	unsigned int	potentional;
	unsigned int	outputvol;
	unsigned int	count;

	OSCCON = 0x70;		// internal OSC = 8MHz
	PORTA = 0x00;       //
	PORTB = 0x00;		//
	ANSEL = 0x07;		//AN2,1,0 = A/D 
	ADCON0 = 0xC0;		//
	ADCON1 = 0x80;		//ADFN=1 Right justified, ADCS2=1, reference=Vdd/Vss
	CMCON = 0x07;		//Comparator setting Not use
 	OPTION_REG = 0x82;
 	TMR0 = 0;
 	TRISA = Amode;		//0x07		//port A initial mode  7,6,5,4,3=output  2,1,0=input
 	TRISB = Bmode;		//0x00		//port B initial mode  7,6,5,4,3,2,1,0=output
	ADON = 1;

//---------------------------------------------
	COM1 = LOW;
	COM2 = LOW;
	COM3 = LOW;
	COM4 = LOW;

	SRCLK = LOW;
	RCLK = LOW;

//---------------------------------------------
		ad(0);
		convert2volt();
		potentional=ch_v;

		ad(1);
		convert2volt();
		outputvol=ch_v;


		if(potentional<160 ){		// 3x1.6=4.8V
			RELAY = LOW;
        }
		else{
			RELAY = HIGH;
        }


		COM1=HIGH;
#if 0
		serialout(leddata[0]);
        seriallatch();
		delay_ms(200);
		serialout(leddata[1]);
        seriallatch();
		delay_ms(200);
		serialout(leddata[2]);
        seriallatch();
		delay_ms(200);
		serialout(leddata[3]);
        seriallatch();
		delay_ms(200);
		serialout(leddata[4]);
        seriallatch();
		delay_ms(200);
		serialout(leddata[5]);
        seriallatch();
		delay_ms(200);
		serialout(leddata[6]);
        seriallatch();
		delay_ms(200);
		serialout(leddata[7]);
        seriallatch();
		delay_ms(200);
		serialout(leddata[8]);
        seriallatch();
		delay_ms(200);
		serialout(leddata[9]);
        seriallatch();
		delay_ms(200);
#endif
		count = 0;
//		COM1 = HIGH;

//---------------------------

		GIE = 1;
		PEIE = 1;

	    TMR1IF = 0;    // Timer1荞݃tO𗎂ƂĂ
    	TMR1IE = 1;    // Timer1荞݂L
    	T1CKPS1 = 0;   // vXP[1/1ɂ
    	T1CKPS0 = 0;   // vXP[1/1ɂ
    	TMR1 = TIMER_INTERVAL;  // ^C}[lZbg܂
    	TMR1ON = 1;   // Timer1Lɂ

//---------------------------------------------------------
	while(1){			//endless loop


//------------------------------A/D convert ch1 1/3output
		TMR1IE = 0;    // Timer1荞݂𖳌

		ad(1);
		convert2volt();
		outputvol=ch_v;
//        ch_v = ch_v * (0.987+0.986+0.0223+0.991)/0.991;
        ch_v = ch_v * 3.013421;
		get_val();		// get 7seg diplay number

//------------------------------A/D convert ch0 potentional
		ad(0);
		convert2volt();
		potentional=ch_v;

		TMR1IE = 1;    // Timer1荞݂L

//--------------------------
//		if(potentional<160 ){		// 3.013421x1.6=4.821474V
		if(potentional<162 ){		// 3.013421x1.62=4.881742V
			RELAY = LOW;
        }
		else if(potentional>166 ){		// 3.013421x1.66=5.002279V
			RELAY = HIGH;
        }

//		count++;
//		if(count==1000){
//			count=0;
//		}

//		delay_ms(19);		// A/D convert cycle = 19msec

	}	// end of while
}	// end of main


//-----------------------------------------------
void __interrupt() intr(void)
{
//	static int intr_counter = 0;

	// Timer1荞݂Ƃ̏
	if ( TMR1IF == 1 ) {
		TMR1 = TIMER_INTERVAL;  // ^C}[JE^ɏlݒ

		// A/D value  val4 val3 . val2 val1
		// 7SEG       COM1 COM2   COM3 COM4

//		intr_counter++;
//		if ( intr_counter == 10 ){		//2msec x 100 = 500msec
//			intr_counter = 0;

			if ( COM1==HIGH ){
				serialout(0x7F & leddata[val3]);	// MSB=DP=LED on
				COM1=LOW;
				seriallatch();
				COM2=HIGH;
				TMR1IF = 0;	// 荞݃tONA
				return;
			}
			if ( COM2==HIGH ){
				serialout(leddata[val2]);
				COM2=LOW;
				seriallatch();
				COM3=HIGH;
				TMR1IF = 0;	// 荞݃tONA
				return;
			}
			if ( COM3==HIGH ){
				serialout(leddata[val1]);
				COM3=LOW;
				seriallatch();
				COM4=HIGH;
				TMR1IF = 0;	// 荞݃tONA
				return;
			}
			if ( COM4==HIGH ){
				if( val4 == 0 ){
					serialout(0x7F & leddata[val3]);	// MSB=DP=LED on
					COM4=LOW;
					seriallatch();
					COM2=HIGH;
				}else{
					serialout(leddata[val4]);
					COM4=LOW;
					seriallatch();
					COM1=HIGH;
				TMR1IF = 0;	// 荞݃tONA
				return;
				}
			}
//		}

		TMR1IF = 0;    // 荞݃tONA

	}

	return;

}

